// Control lab simulator
// Glenn Vinnicombe, November 2006
// (based on template code by Andrew Gee)

#ifdef WIN32
#include <stdlib.h>
#endif
#include <GL/glut.h>
#include <iostream>
#include <fstream>
#include <malloc.h>
#ifdef WIN32
#define _USE_MATH_DEFINES
#include <math.h>
#include <windows.h>
#else
#include <cmath>
#include <sys/time.h>
#endif

#include <complex>

#define GAP 5
#define PART_PIXEL 0.1
#define PREFERRED_WIDTH 1024
#define PREFERRED_HEIGHT 768
#define AXIS_OFFSET 30.0
#define REF_Y_ORIGIN 120.0
#define REF_Y_SCALE 10.0
#define ERROR_Y_ORIGIN 120.0
#define ERROR_Y_SCALE 10.0
#define CLFR_MAG_ORIGIN 120.0
#define CLFR_MAG_SCALE 10.0
#define BODE_MAG_ORIGIN 180.0
#define BODE_MAG_SCALE 1.0
#define C_BODE_MAG_ORIGIN 165.0
#define C_BODE_MAG_SCALE 1.0
#define CLFR_ORIGIN 150.0
#define CLFR_SCALE 2.0
#define BODE_PH_ORIGIN 100.0
#define BODE_PH_SCALE .33
#define C_BODE_PH_ORIGIN 60.0
#define C_BODE_PH_SCALE .33
#define NYQUIST_X_ORIGIN 150.0
#define NYQUIST_Y_ORIGIN 120.0
#define NYQUIST_SCALE 75.0
#define RLOCUS_X_ORIGIN 150.0
#define RLOCUS_Y_ORIGIN 120.0
#define RLOCUS_SCALE 60.0
#define G_X_POS 250
#define K_X_POS 160
#define OM_X_POS 20
#define S_X_POS 90
#define G_Y_POS 150
#define G_X_SIZE 25
#define G_Y_SIZE 15
#define Y_LOOP 50


using namespace std;

complex<float> foo;

float tnow = 0 ;
float delta_t = 0.1;
float theta = 0;

// Principal parameters
float tau = 20;
float kp = 1.0;
float kp_max = 20.0;
float kpdb = 0;
//float kcrit=2.2;
float kcrit=100;
float omega = 0.3;
float signal_voltage = 0.0;
float signal_voltage_orig = 0.0;
float num[3];
float den[4];
float numc[2];
float denc[2];
float numl[4];
float denl[5];




// GLUT windows
int main_window, characteristic_window, ref_window, circuit_window, error_window, nyquist_window, bode_window, clfr_window, inst_window, cont_window;
int subwin_width, subwin_height, win_width, win_height;

// Simulation status
bool adjusting_om_max = false;
bool adjusting_klf = false;
bool adjusting_ref = false;
bool adjusting_kp = false;
bool adjusting_omega = false;
bool paused = false;
bool sinusoid = false;
bool params_changed=true;
bool refresh_circuit=true;
bool controller =false;
int last_click_y = -1;
int last_click_x = -1;
int millisecond_delay = 10;

// Time traces 
int most_recent = 0;
float historical_ref[400];
float historical_u[400];
float historical_yout[400];
float historical_error[400];

complex<float> frsp[151];
complex<float> frspw;
float frsp_mag[151];
float frsp_ph[151];
float frsp_real[151];
float frsp_imag[151];

complex<float> kfrsp[151];
complex<float> kfrspw;
float kfrsp_mag[151];
float kfrsp_ph[151];

float klf=.3;
float d_con;
float b_con;
float c_con;
float om_max=1.0;

/*      rpoly.cpp -- Jenkins-Traub real polynomial root finder.
 *
 *      (C) 2000, C. Bond.  All rights reserved.
 *
 *      Translation of TOMS493 from FORTRAN to C. This
 *      implementation of Jenkins-Traub partially adapts
 *      the original code to a C environment by restruction
 *      many of the 'goto' controls to better fit a block
 *      structured form. It also eliminates the global memory
 *      allocation in favor of local, dynamic memory management.
 *
 *      The calling conventions are slightly modified to return
 *      the number of roots found as the function value.
 *
 *      INPUT:
 *      op - double precision vector of coefficients in order of
 *              decreasing powers.
 *      degree - integer degree of polynomial
 *
 *      OUTPUT:
 *      zeror,zeroi - output double precision vectors of the
 *              real and imaginary parts of the zeros.
 *
 *      RETURN:
 *      returnval:   -1 if leading coefficient is zero, otherwise
 *                  number of roots found. 
 */

#include "math.h"

void quad(double a,double b1,double c,double *sr,double *si,
        double *lr,double *li);
void fxshfr(int l2, int *nz);
void quadit(double *uu,double *vv,int *nz);
void realit(double *sss, int *nz, int *iflag);
void calcsc(int *type);
void nextk(int *type);
void newest(int type,double *uu,double *vv);
void quadsd(int n,double *u,double *v,double *p,double *q,
        double *a,double *b);
double *p,*qp,*k,*qk,*svk;
double sr,si,u,v,a,b,c,d,a1,a2;
double a3,a6,a7,e,f,g,h,szr,szi,lzr,lzi;
double eta,are,mre;
int n,nn,nmi,zerok;

int rpoly(double *op, int degree, double *zeror, double *zeroi) 
{
    double t,aa,bb,cc,*temp,factor,rot;
    double *pt;
    double lo,max,min,xx,yy,cosr,sinr,xxx,x,sc,bnd;
    double xm,ff,df,dx,infin,smalno,base;
    int cnt,nz,i,j,jj,l,nm1,zerok;
/*  The following statements set machine constants. */
    base = 2.0;
    eta = 2.22e-16;
    infin = 3.4e38;
    smalno = 1.2e-38;

    are = eta;
    mre = eta;
    lo = smalno/eta;
/*  Initialization of constants for shift rotation. */        
    xx = sqrt(0.5);
    yy = -xx;
    rot = 94.0;
    rot *= 0.017453293;
    cosr = cos(rot);
    sinr = sin(rot);
    n = degree;
/*  Algorithm fails of the leading coefficient is zero. */
    if (op[0] == 0.0) return -1;
/*  Remove the zeros at the origin, if any. */
    while (op[n] == 0.0) {
        j = degree - n;
        zeror[j] = 0.0;
        zeroi[j] = 0.0;
        n--;
    }
    if (n < 1) return -1;
/*
 *  Allocate memory here
 */
    temp = new double [degree+1];
    pt = new double [degree+1];
    p = new double [degree+1];
    qp = new double [degree+1];
    k = new double [degree+1];
    qk = new double [degree+1];
    svk = new double [degree+1];
/*  Make a copy of the coefficients. */
    for (i=0;i<=n;i++)
        p[i] = op[i];
/*  Start the algorithm for one zero. */
_40:        
    if (n == 1) {
        zeror[degree-1] = -p[1]/p[0];
        zeroi[degree-1] = 0.0;
        n -= 1;
        goto _99;
    }
/*  Calculate the final zero or pair of zeros. */
    if (n == 2) {
        quad(p[0],p[1],p[2],&zeror[degree-2],&zeroi[degree-2],
            &zeror[degree-1],&zeroi[degree-1]);
        n -= 2;
        goto _99;
    }
/*  Find largest and smallest moduli of coefficients. */
    max = 0.0;
    min = infin;
    for (i=0;i<=n;i++) {
        x = fabs(p[i]);
        if (x > max) max = x;
        if (x != 0.0 && x < min) min = x;
    }
/*  Scale if there are large or very small coefficients.
 *  Computes a scale factor to multiply the coefficients of the
 *  polynomial. The scaling si done to avoid overflow and to
 *  avoid undetected underflow interfering with the convergence
 *  criterion. The factor is a power of the base.
 */
    sc = lo/min;
    if (sc > 1.0 && infin/sc < max) goto _110;
    if (sc <= 1.0) {
        if (max < 10.0) goto _110;
        if (sc == 0.0)
            sc = smalno;
    }
    l = (int)(log(sc)/log(base) + 0.5);
    factor = pow(base*1.0,l);
    if (factor != 1.0) {
        for (i=0;i<=n;i++) 
            p[i] = factor*p[i];     /* Scale polynomial. */
    }
_110:
/*  Compute lower bound on moduli of roots. */
    for (i=0;i<=n;i++) {
        pt[i] = (fabs(p[i]));
    }
    pt[n] = - pt[n];
/*  Compute upper estimate of bound. */
    x = exp((log(-pt[n])-log(pt[0])) / (double)n);
/*  If Newton step at the origin is better, use it. */        
    if (pt[n-1] != 0.0) {
        xm = -pt[n]/pt[n-1];
        if (xm < x)  x = xm;
    }
/*  Chop the interval (0,x) until ff <= 0 */
    while (1) {
        xm = x*0.1;
        ff = pt[0];
        for (i=1;i<=n;i++) 
            ff = ff*xm + pt[i];
        if (ff <= 0.0) break;
        x = xm;
    }
    dx = x;
/*  Do Newton interation until x converges to two 
 *  decimal places. 
 */
    while (fabs(dx/x) > 0.005) {
        ff = pt[0];
        df = ff;
        for (i=1;i<n;i++) { 
            ff = ff*x + pt[i];
            df = df*x + ff;
        }
        ff = ff*x + pt[n];
        dx = ff/df;
        x -= dx;
    }
    bnd = x;
/*  Compute the derivative as the initial k polynomial
 *  and do 5 steps with no shift.
 */
    nm1 = n - 1;
    for (i=1;i<n;i++)
        k[i] = (double)(n-i)*p[i]/(double)n;
    k[0] = p[0];
    aa = p[n];
    bb = p[n-1];
    zerok = (k[n-1] == 0);
    for(jj=0;jj<5;jj++) {
        cc = k[n-1];
        if (!zerok) {
/*  Use a scaled form of recurrence if value of k at 0 is nonzero. */             
            t = -aa/cc;
            for (i=0;i<nm1;i++) {
                j = n-i-1;
                k[j] = t*k[j-1]+p[j];
            }
            k[0] = p[0];
            zerok = (fabs(k[n-1]) <= fabs(bb)*eta*10.0);
        }
        else {
/*  Use unscaled form of recurrence. */
            for (i=0;i<nm1;i++) {
                j = n-i-1;
                k[j] = k[j-1];
            }
            k[0] = 0.0;
            zerok = (k[n-1] == 0.0);
        }
    }
/*  Save k for restarts with new shifts. */
    for (i=0;i<n;i++) 
        temp[i] = k[i];
/*  Loop to select the quadratic corresponding to each new shift. */
    for (cnt = 0;cnt < 20;cnt++) {
/*  Quadratic corresponds to a double shift to a            
 *  non-real point and its complex conjugate. The point
 *  has modulus bnd and amplitude rotated by 94 degrees
 *  from the previous shift.
 */ 
        xxx = cosr*xx - sinr*yy;
        yy = sinr*xx + cosr*yy;
        xx = xxx;
        sr = bnd*xx;
        si = bnd*yy;
        u = -2.0 * sr;
        v = bnd;
        fxshfr(20*(cnt+1),&nz);
        if (nz != 0) {
/*  The second stage jumps directly to one of the third
 *  stage iterations and returns here if successful.
 *  Deflate the polynomial, store the zero or zeros and
 *  return to the main algorithm.
 */
            j = degree - n;
            zeror[j] = szr;
            zeroi[j] = szi;
            n -= nz;
            for (i=0;i<=n;i++)
                p[i] = qp[i];
            if (nz != 1) {
                zeror[j+1] = lzr;
                zeroi[j+1] = lzi;
            }
            goto _40;
        }
/*  If the iteration is unsuccessful another quadratic
 *  is chosen after restoring k.
 */
        for (i=0;i<n;i++) {
            k[i] = temp[i];
        }
    } 
/*  Return with failure if no convergence with 20 shifts. */
_99:
    delete [] svk;
    delete [] qk;
    delete [] k;
    delete [] qp;
    delete [] p;
    delete [] pt;
    delete [] temp;

    return degree - n;
}
/*  Computes up to L2 fixed shift k-polynomials,
 *  testing for convergence in the linear or quadratic
 *  case. Initiates one of the variable shift
 *  iterations and returns with the number of zeros
 *  found.
 */
void fxshfr(int l2,int *nz)
{
    double svu,svv,ui,vi,s;
    double betas,betav,oss,ovv,ss,vv,ts,tv;
    double ots,otv,tvv,tss;
	int type, i,j,iflag,vpass,spass,vtry,stry;

	*nz = 0;
	betav = 0.25;
	betas = 0.25;
	oss = sr;
	ovv = v;
/*  Evaluate polynomial by synthetic division. */
    quadsd(n,&u,&v,p,qp,&a,&b);
	calcsc(&type);
	for (j=0;j<l2;j++) {
/*  Calculate next k polynomial and estimate v. */
		nextk(&type);
		calcsc(&type);
		newest(type,&ui,&vi);
		vv = vi;
/*  Estimate s. */
        ss = 0.0;
        if (k[n-1] != 0.0) ss = -p[n]/k[n-1];
		tv = 1.0;
		ts = 1.0;
		if (j == 0 || type == 3) goto _70;
/*  Compute relative measures of convergence of s and v sequences. */
        if (vv != 0.0) tv = fabs((vv-ovv)/vv);
        if (ss != 0.0) ts = fabs((ss-oss)/ss);
/*  If decreasing, multiply two most recent convergence measures. */
		tvv = 1.0;
		if (tv < otv) tvv = tv*otv;
		tss = 1.0;
		if (ts < ots) tss = ts*ots;
/*  Compare with convergence criteria. */
		vpass = (tvv < betav);
		spass = (tss < betas);
		if (!(spass || vpass)) goto _70;
/*  At least one sequence has passed the convergence test.
 *  Store variables before iterating.
 */
		svu = u;
		svv = v;
		for (i=0;i<n;i++) {
			svk[i] = k[i];
		}
		s = ss;
/*  Choose iteration according to the fastest converging
 *  sequence.
 */
		vtry = 0;
		stry = 0;
		if (spass && (!vpass) || tss < tvv) goto _40;
_20:        
		quadit(&ui,&vi,nz);
        if (*nz > 0) return;
/*  Quadratic iteration has failed. Flag that it has
 *  been tried and decrease the convergence criterion.
 */
		vtry = 1;
		betav *= 0.25;
/*  Try linear iteration if it has not been tried and
 *  the S sequence is converging.
 */
		if (stry || !spass) goto _50;
		for (i=0;i<n;i++) {
			k[i] = svk[i];
		}
_40:
        realit(&s,nz,&iflag);
		if (*nz > 0) return;
/*  Linear iteration has failed. Flag that it has been
 *  tried and decrease the convergence criterion.
 */
		stry = 1;
		betas *=0.25;
		if (iflag == 0) goto _50;
/*  If linear iteration signals an almost double real
 *  zero attempt quadratic iteration.
 */
		ui = -(s+s);
		vi = s*s;
		goto _20;
/*  Restore variables. */
_50:
		u = svu;
		v = svv;
		for (i=0;i<n;i++) {
			k[i] = svk[i];
		}
/*  Try quadratic iteration if it has not been tried
 *  and the V sequence is convergin.
 */
		if (vpass && !vtry) goto _20;
/*  Recompute QP and scalar values to continue the
 *  second stage.
 */
        quadsd(n,&u,&v,p,qp,&a,&b);
		calcsc(&type);
_70:
		ovv = vv;
		oss = ss;
		otv = tv;
		ots = ts;
	}
}
/*  Variable-shift k-polynomial iteration for a
 *  quadratic factor converges only if the zeros are
 *  equimodular or nearly so.
 *  uu, vv - coefficients of starting quadratic.
 *  nz - number of zeros found.
 */
void quadit(double *uu,double *vv,int *nz)
{
    double ui,vi;
    double mp,omp,ee,relstp,t,zm;
	int type,i,j,tried;

	*nz = 0;
	tried = 0;
	u = *uu;
	v = *vv;
	j = 0;
/*  Main loop. */
_10:    
	quad(1.0,u,v,&szr,&szi,&lzr,&lzi);
/*  Return if roots of the quadratic are real and not
 *  close to multiple or nearly equal and of opposite
 *  sign.
 */
    if (fabs(fabs(szr)-fabs(lzr)) > 0.01 * fabs(lzr)) return;
/*  Evaluate polynomial by quadratic synthetic division. */
    quadsd(n,&u,&v,p,qp,&a,&b);
    mp = fabs(a-szr*b) + fabs(szi*b);
/*  Compute a rigorous bound on the rounding error in
 *  evaluating p.
 */
    zm = sqrt(fabs(v));
    ee = 2.0*fabs(qp[0]);
	t = -szr*b;
	for (i=1;i<n;i++) {
        ee = ee*zm + fabs(qp[i]);
	}
    ee = ee*zm + fabs(a+t);
    ee *= (5.0 *mre + 4.0*are);
       ee = ee - (5.0*mre+2.0*are)*(fabs(a+t)+fabs(b)*zm)+2.0*are*fabs(t);
/*  Iteration has converged sufficiently if the
 *  polynomial value is less than 20 times this bound.
 */
    if (mp <= 20.0*ee) {
        *nz = 2;
        return;
    }
	j++;
/*  Stop iteration after 20 steps. */
	if (j > 20) return;
	if (j < 2) goto _50;
    if (relstp > 0.01 || mp < omp || tried) goto _50;
/*  A cluster appears to be stalling the convergence.
 *  Five fixed shift steps are taken with a u,v close
 *  to the cluster.
 */
	if (relstp < eta) relstp = eta;
	relstp = sqrt(relstp);
	u = u - u*relstp;
	v = v + v*relstp;
    quadsd(n,&u,&v,p,qp,&a,&b);
	for (i=0;i<5;i++) {
		calcsc(&type);
		nextk(&type);
	}
	tried = 1;
	j = 0;
_50:
	omp = mp;
/*  Calculate next k polynomial and new u and v. */
	calcsc(&type);
	nextk(&type);
	calcsc(&type);
	newest(type,&ui,&vi);
/*  If vi is zero the iteration is not converging. */
    if (vi == 0.0) return;
    relstp = fabs((vi-v)/vi);
	u = ui;
	v = vi;
	goto _10;
}
/*  Variable-shift H polynomial iteration for a real zero.
 *  sss - starting iterate
 *  nz  - number of zeros found
 *  iflag - flag to indicate a pair of zeros near real axis.
 */
void realit(double *sss, int *nz, int *iflag)
{
    double pv,kv,t,s;
    double ms,mp,omp,ee;
    int i,j;

    *nz = 0;
    s = *sss;
    *iflag = 0;
	j = 0;
/*  Main loop */
    while (1) {
        pv = p[0];
/*  Evaluate p at s. */
        qp[0] = pv;
        for (i=1;i<=n;i++) {
            pv = pv*s + p[i];
            qp[i] = pv;
        }
        mp = fabs(pv);
/*  Compute a rigorous bound on the error in evaluating p. */
        ms = fabs(s);
        ee = (mre/(are+mre))*fabs(qp[0]);
        for (i=1;i<=n;i++) {
            ee = ee*ms + fabs(qp[i]);
        }
/*  Iteration has converged sufficiently if the polynomial
 *  value is less than 20 times this bound.
 */
        if (mp <= 20.0*((are+mre)*ee-mre*mp)) {
            *nz = 1;
            szr = s;
            szi = 0.0;
            return;
        }
        j++;
/*  Stop iteration after 10 steps. */
        if (j > 10) return;
        if (j < 2) goto _50;
        if (fabs(t) > 0.001*fabs(s-t) || mp < omp) goto _50;
/*  A cluster of zeros near the real axis has been
 *  encountered. Return with iflag set to initiate a
 *  quadratic iteration.
 */
        *iflag = 1;
        *sss = s;
        return;
/*  Return if the polynomial value has increased significantly. */
_50:
        omp = mp;
/*  Compute t, the next polynomial, and the new iterate. */
        kv = k[0];
        qk[0] = kv;
        for (i=1;i<n;i++) {
            kv = kv*s + k[i];
            qk[i] = kv;
        }
        if (fabs(kv) <= fabs(k[n-1])*10.0*eta) {
/*  Use unscaled form. */
            k[0] = 0.0;
            for (i=1;i<n;i++) {
                k[i] = qk[i-1];
            }
        }
        else {
/*  Use the scaled form of the recurrence if the value
 *  of k at s is nonzero.
 */
            t = -pv/kv;
            k[0] = qp[0];
            for (i=1;i<n;i++) {
                k[i] = t*qk[i-1] + qp[i];
            }
        }
        kv = k[0];
        for (i=1;i<n;i++) {
            kv = kv*s + k[i];
        }
        t = 0.0;
        if (fabs(kv) > fabs(k[n-1]*10.0*eta)) t = -pv/kv;
        s += t;
    }
}

/*  This routine calculates scalar quantities used to
 *  compute the next k polynomial and new estimates of
 *  the quadratic coefficients.
 *  type - integer variable set here indicating how the
 *  calculations are normalized to avoid overflow.
 */
void calcsc(int *type)
{
/*  Synthetic division of k by the quadratic 1,u,v */    
    quadsd(n-1,&u,&v,k,qk,&c,&d);
    if (fabs(c) > fabs(k[n-1]*100.0*eta)) goto _10;
    if (fabs(d) > fabs(k[n-2]*100.0*eta)) goto _10;
	*type = 3;
/*  Type=3 indicates the quadratic is almost a factor of k. */
	return;
_10:
    if (fabs(d) < fabs(c)) {
        *type = 1;
/*  Type=1 indicates that all formulas are divided by c. */   
        e = a/c;
        f = d/c;
        g = u*e;
        h = v*b;
        a3 = a*e + (h/c+g)*b;
        a1 = b - a*(d/c);
        a7 = a + g*d + h*f;
        return;
    }
    *type = 2;
/*  Type=2 indicates that all formulas are divided by d. */
	e = a/d;
	f = c/d;
	g = u*b;
	h = v*b;
	a3 = (a+g)*e + h*(b/d);
	a1 = b*f-a;
	a7 = (f+u)*a + h;
}
/*  Computes the next k polynomials using scalars 
 *  computed in calcsc.
 */
void nextk(int *type)
{
    double temp;
	int i;

    if (*type == 3) {
/*  Use unscaled form of the recurrence if type is 3. */
        k[0] = 0.0;
        k[1] = 0.0;
        for (i=2;i<n;i++) {
            k[i] = qk[i-2];
        }
        return;
    }
	temp = a;
	if (*type == 1) temp = b;
    if (fabs(a1) <= fabs(temp)*eta*10.0) {
/*  If a1 is nearly zero then use a special form of the
 *  recurrence.
 */
        k[0] = 0.0;
        k[1] = -a7*qp[0];
        for(i=2;i<n;i++) {
            k[i] = a3*qk[i-2] - a7*qp[i-1];
        }
        return;
    }
/*  Use scaled form of the recurrence. */
	a7 /= a1;
	a3 /= a1;
	k[0] = qp[0];
	k[1] = qp[1] - a7*qp[0];
	for (i=2;i<n;i++) {
		k[i] = a3*qk[i-2] - a7*qp[i-1] + qp[i];
	}
}
/*  Compute new estimates of the quadratic coefficients
 *  using the scalars computed in calcsc.
 */
void newest(int type,double *uu,double *vv)
{
    double a4,a5,b1,b2,c1,c2,c3,c4,temp;

/* Use formulas appropriate to setting of type. */
    if (type == 3) {
/*  If type=3 the quadratic is zeroed. */
        *uu = 0.0;
        *vv = 0.0;
        return;
    }
    if (type == 2) {
        a4 = (a+g)*f + h;
        a5 = (f+u)*c + v*d;
    }
    else {
        a4 = a + u*b +h*f;
        a5 = c + (u+v*f)*d;
    }
/*  Evaluate new quadratic coefficients. */
    b1 = -k[n-1]/p[n];
    b2 = -(k[n-2]+b1*p[n-1])/p[n];
	c1 = v*b2*a1;
	c2 = b1*a7;
	c3 = b1*b1*a3;
	c4 = c1 - c2 - c3;
	temp = a5 + b1*a4 - c4;
    if (temp == 0.0) {
        *uu = 0.0;
        *vv = 0.0;
        return;
    }
	*uu = u - (u*(c3+c2)+v*(b1*a1+b2*a7))/temp;
	*vv = v*(1.0+c4/temp);
	return;
}

/*  Divides p by the quadratic 1,u,v placing the quotient
 *  in q and the remainder in a,b.
 */
void quadsd(int nn,double *u,double *v,double *p,double *q,
    double *a,double *b)
{
    double c;
	int i;
	*b = p[0];
	q[0] = *b;
    *a = p[1] - (*b)*(*u);
	q[1] = *a;
    for (i=2;i<=nn;i++) {
        c = p[i] - (*a)*(*u) - (*b)*(*v);
		q[i] = c;
		*b = *a;
		*a = c;
	}
}
/*  Calculate the zeros of the quadratic a*z^2 + b1*z + c.
 *  The quadratic formula, modified to avoid overflow, is used 
 *  to find the larger zero if the zeros are real and both
 *  are complex. The smaller real zero is found directly from 
 *  the product of the zeros c/a.
 */
void quad(double a,double b1,double c,double *sr,double *si,
        double *lr,double *li)
{
        double b,d,e;

        if (a == 0.0) {         /* less than two roots */
            if (b1 != 0.0)     
				*sr = -c/b1;
			else 
                *sr = 0.0;
            *lr = 0.0;
            *si = 0.0;
            *li = 0.0;
			return;
		}
        if (c == 0.0) {         /* one real root, one zero root */
            *sr = 0.0;
			*lr = -b1/a;
            *si = 0.0;
            *li = 0.0;
			return;
		}
/* Compute discriminant avoiding overflow. */
		b = b1/2.0;
        if (fabs(b) < fabs(c)) { 
            if (c < 0.0) 
				e = -a;
			else
				e = a;
            e = b*(b/fabs(c)) - e;
            d = sqrt(fabs(e))*sqrt(fabs(c));
		}
		else {
			e = 1.0 - (a/b)*(c/b);
            d = sqrt(fabs(e))*fabs(b);
		}
        if (e < 0.0) {      /* complex conjugate zeros */
			*sr = -b/a;
			*lr = *sr;
            *si = fabs(d/a);
			*li = -(*si);
		}
		else {
            if (b >= 0.0)   /* real zeros. */
				d = -d;
				*lr = (-b+d)/a;
                *sr = 0.0;
                if (*lr != 0.0) 
					*sr = (c/ *lr)/a;
                *si = 0.0;
                *li = 0.0;
		}
}
void glut_print (float x, float y, char *string)
  // Prints string at location (x,y) in a bitmap font
{
  int len, i;

  len = (int) strlen(string);
  glRasterPos2f(x, y);
  for (i = 0; i < len; i++) glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12, string[i]);
}

void draw_ref_tick (int y)
  // Draws a tick mark and value on the y-axis of the Ref graph
{
  char mess[8];
  glColor3f(1.0, 1.0, 1.0);
  glBegin(GL_LINES);
  glVertex2f(AXIS_OFFSET, REF_Y_ORIGIN + REF_Y_SCALE*y);
  glVertex2f(AXIS_OFFSET-5, REF_Y_ORIGIN + REF_Y_SCALE*y);
  glEnd();
  sprintf(mess,"%d",y);
  glut_print(AXIS_OFFSET-15-7*(strlen(mess)-1), REF_Y_ORIGIN + REF_Y_SCALE*y - 4, mess);
}

void draw_ref_window (void)
  // Draws the Ref graph
{
  int i, j;
  float ref, yout, error, u;
  GLushort old_stipple;
  static GLushort stipple = 0xFF00;

  glutSetWindow(ref_window);
  glClear(GL_COLOR_BUFFER_BIT);

 // Record the operating point in the historical record
  ref = signal_voltage;
  most_recent = (most_recent + 1) % 300;
  historical_ref[most_recent] = ref;

  if (!controller) 
    {
      yout=  -den[1]*historical_yout[ (most_recent-1+300) % 300 ];
      yout+= -den[2]*historical_yout[ (most_recent-2+300) % 300 ];
      yout+= -den[3]*historical_yout[ (most_recent-3+300) % 300 ];
      yout+=  num[0]* historical_u[ (most_recent-1+300) % 300 ];
      yout+=  num[1]* historical_u[ (most_recent-2+300) % 300 ];
      yout+=  num[2]* historical_u[ (most_recent-3+300) % 300 ];
    }
  else
    {
      yout=  -denl[1]*historical_yout[ (most_recent-1+300) % 300 ];
      yout+= -denl[2]*historical_yout[ (most_recent-2+300) % 300 ];
      yout+= -denl[3]*historical_yout[ (most_recent-3+300) % 300 ];
      yout+= -denl[4]*historical_yout[ (most_recent-4+300) % 300 ];
      yout+=  numl[0]* historical_u[ (most_recent-1+300) % 300 ];
      yout+=  numl[1]* historical_u[ (most_recent-2+300) % 300 ];
      yout+=  numl[2]* historical_u[ (most_recent-3+300) % 300 ];
      yout+=  numl[3]* historical_u[ (most_recent-4+300) % 300 ];
    }
  historical_yout[most_recent] = yout;
  error=ref-yout;
  u=kp*error;
  historical_u[most_recent] = u;
  historical_error[most_recent] = error;


  // Draw axis, label and ticks
  glColor3f(1.0, 1.0, 1.0);
  glBegin(GL_LINES);
  glVertex2f(AXIS_OFFSET, REF_Y_ORIGIN + REF_Y_SCALE*(-10.0));
  glVertex2f(AXIS_OFFSET, REF_Y_ORIGIN + REF_Y_SCALE*(10.0));
  glEnd();
  glColor3f(1.0, 0.0, 1.0);
  glut_print(AXIS_OFFSET-25.0,REF_Y_ORIGIN + REF_Y_SCALE*(11.0),"ref(t)");
  glColor3f(1.0, 1.0, 1.0);
  glut_print(AXIS_OFFSET+3,REF_Y_ORIGIN + REF_Y_SCALE*(11.0),"and");
  glColor3f(0.0, 1.0, 1.0);
  glut_print(AXIS_OFFSET+28.0,REF_Y_ORIGIN + REF_Y_SCALE*(11.0),"y(t)");
  draw_ref_tick(0); draw_ref_tick(-5);draw_ref_tick(-10); 
  draw_ref_tick(10); draw_ref_tick(5);


  // Draw the historical Yout trace
  glColor3f(0.0, 1.0, 1.0);
  glBegin(GL_LINE_STRIP);
  j = 0;
  for (i=most_recent; i>=0; i--) {
    glVertex2f(AXIS_OFFSET +300 - j, REF_Y_ORIGIN + REF_Y_SCALE*historical_yout[i]);
    j++;
  }
  for (i=299; i>most_recent; i--) {
    glVertex2f(AXIS_OFFSET +300 - j, REF_Y_ORIGIN + REF_Y_SCALE*historical_yout[i]);
    j++;
  }
  glEnd();

  // Draw the historical Ref trace
  glColor3f(1.0, 0.0, 1.0);
  glBegin(GL_LINE_STRIP);
  j = 0;
  for (i=most_recent; i>=0; i--) {
    glVertex2f(AXIS_OFFSET +300 - j, REF_Y_ORIGIN + REF_Y_SCALE*historical_ref[i]);
    j++;
  }
  for (i=299; i>most_recent; i--) {
    glVertex2f(AXIS_OFFSET +300 - j, REF_Y_ORIGIN + REF_Y_SCALE*historical_ref[i]);
    j++;
  }
  glEnd();


  glutSwapBuffers();
}

void draw_error_tick (int y)
  // Draws a tick mark and value on the y-axis of the Error graph
{
  char mess[8];
  glColor3f(1.0, 1.0, 1.0);
  glBegin(GL_LINES);
  glVertex2f(AXIS_OFFSET, ERROR_Y_ORIGIN + ERROR_Y_SCALE*y);
  glVertex2f(AXIS_OFFSET-5, ERROR_Y_ORIGIN + ERROR_Y_SCALE*y);
  glEnd();
  sprintf(mess,"%d",y);
  glut_print(AXIS_OFFSET-15-7*(strlen(mess)-1), ERROR_Y_ORIGIN + ERROR_Y_SCALE*y - 4, mess);
}

void draw_bode_mag_tick (int y)
  // Draws a tick mark and value on the y-axis of the Error graph
{
  char mess[8];
  glColor3f(1.0, 1.0, 1.0);
  glBegin(GL_LINES);
  glVertex2f(AXIS_OFFSET, BODE_MAG_ORIGIN + BODE_MAG_SCALE*y);
  glVertex2f(AXIS_OFFSET-5, BODE_MAG_ORIGIN + BODE_MAG_SCALE*y);
  glEnd();
  sprintf(mess,"%d",y);
  glut_print(AXIS_OFFSET-15-7*(strlen(mess)-1), BODE_MAG_ORIGIN + BODE_MAG_SCALE*y - 4, mess);
}

void draw_c_bode_mag_tick (int y)
  // Draws a tick mark and value on the y-axis of the Error graph
{
  char mess[8];
  glColor3f(1.0, 1.0, 1.0);
  glBegin(GL_LINES);
  glVertex2f(AXIS_OFFSET, C_BODE_MAG_ORIGIN + C_BODE_MAG_SCALE*y);
  glVertex2f(AXIS_OFFSET-5, C_BODE_MAG_ORIGIN + C_BODE_MAG_SCALE*y);
  glEnd();
  sprintf(mess,"%d",y);
  glut_print(AXIS_OFFSET-15-7*(strlen(mess)-1), C_BODE_MAG_ORIGIN + C_BODE_MAG_SCALE*y - 4, mess);
}

void draw_c_bode_ph_tick (int y)
  // Draws a tick mark and value on the y-axis of the Error graph
{
  char mess[8];
  float i;
  glColor3f(1.0, 1.0, 1.0);
  glBegin(GL_LINES);
  glVertex2f(AXIS_OFFSET, C_BODE_PH_ORIGIN + C_BODE_PH_SCALE*y);
  glVertex2f(AXIS_OFFSET-5, C_BODE_PH_ORIGIN + C_BODE_PH_SCALE*y);
  glEnd();
  sprintf(mess,"%d",y);
  i=AXIS_OFFSET-15-7*(strlen(mess)-1);
  if (i<0) i=0;
  glut_print(i, C_BODE_PH_ORIGIN + C_BODE_PH_SCALE*y - 4, mess);
}

void draw_clfr_tick (int y)
  // Draws a tick mark and value on the y-axis of the Error graph
{
  char mess[8];
  glColor3f(1.0, 1.0, 1.0);
  glBegin(GL_LINES);
  glVertex2f(AXIS_OFFSET, CLFR_ORIGIN + CLFR_SCALE*y);
  glVertex2f(AXIS_OFFSET-5, CLFR_ORIGIN + CLFR_SCALE*y);
  glEnd();
  sprintf(mess,"%d",y);
  glut_print(AXIS_OFFSET-15-7*(strlen(mess)-1), CLFR_ORIGIN + CLFR_SCALE*y - 4, mess);
}

void draw_bode_ph_tick (int y)
  // Draws a tick mark and value on the y-axis of the Error graph
{
  char mess[8];
  float i;
  glColor3f(1.0, 1.0, 1.0);
  glBegin(GL_LINES);
  glVertex2f(AXIS_OFFSET, BODE_PH_ORIGIN + BODE_PH_SCALE*y);
  glVertex2f(AXIS_OFFSET-5, BODE_PH_ORIGIN + BODE_PH_SCALE*y);
  glEnd();
  sprintf(mess,"%d",y);
  i=AXIS_OFFSET-15-7*(strlen(mess)-1);
  if (i<0) i=0;
  glut_print(i, BODE_PH_ORIGIN + BODE_PH_SCALE*y - 4, mess);
}


void draw_error_window (void)
  // Draws the Error graph
{
  int i, j;
  GLushort old_stipple;
  static GLushort stipple = 0xFF00;

  glutSetWindow(error_window);
  glClear(GL_COLOR_BUFFER_BIT);

  // Draw axis, label and ticks
  glColor3f(1.0, 1.0, 1.0);
  glBegin(GL_LINES);
  glVertex2f(AXIS_OFFSET, ERROR_Y_ORIGIN + ERROR_Y_SCALE*(-10.0));
  glVertex2f(AXIS_OFFSET, ERROR_Y_ORIGIN + ERROR_Y_SCALE*(10.0));
  glEnd();
  glColor3f(1.0, 1.0, 0.0);  glut_print(AXIS_OFFSET-25.0,ERROR_Y_ORIGIN + ERROR_Y_SCALE*(11.0),"error(t)");
  glColor3f(1.0, 1.0, 1.0);
  draw_error_tick(0); draw_error_tick(-5);draw_error_tick(-10); 
  draw_error_tick(10); draw_error_tick(5);


  // Draw the historical Error trace
  glColor3f(1.0, 1.0, 0.0);
  glBegin(GL_LINE_STRIP);
  j = 0;
  for (i=most_recent; i>=0; i--) {
    glVertex2f(AXIS_OFFSET + 300 - j, ERROR_Y_ORIGIN + ERROR_Y_SCALE*historical_error[i]);
    j++;
  }
  for (i=299; i>most_recent; i--) {
    glVertex2f(AXIS_OFFSET + 300 - j, ERROR_Y_ORIGIN + ERROR_Y_SCALE*historical_error[i]);
    j++;
  }
  glEnd();

  glutSwapBuffers();
}

void draw_bode_window (void)
  // Draws the Bode graph
{
  int i, j;
  float ph;
  GLushort old_stipple;
  static GLushort stipple = 0xFF00;

  glutSetWindow(bode_window);
  glClear(GL_COLOR_BUFFER_BIT);

  // Draw axis, label and ticks
  glColor3f(1.0, 1.0, 1.0);
  glBegin(GL_LINES);
  glVertex2f(AXIS_OFFSET, BODE_MAG_ORIGIN + BODE_MAG_SCALE*(-60.0));
  glVertex2f(AXIS_OFFSET, BODE_MAG_ORIGIN + BODE_MAG_SCALE*(20.0));
  glVertex2f(AXIS_OFFSET, BODE_MAG_ORIGIN + BODE_MAG_SCALE*(0.0));
  glVertex2f(AXIS_OFFSET+300, BODE_MAG_ORIGIN + BODE_MAG_SCALE*(0.0));
  glEnd();
  glut_print(AXIS_OFFSET+0,BODE_MAG_ORIGIN + BODE_MAG_SCALE*(40.0),"Open-loop Bode diagram kp G(jw) :  [ y(jw) / error(jw) ]");
  glut_print(AXIS_OFFSET-25.0,BODE_MAG_ORIGIN + BODE_MAG_SCALE*(20.0)+15,"dB");
  draw_bode_mag_tick(0);   draw_bode_mag_tick(-20); 
  draw_bode_mag_tick(20); draw_bode_mag_tick(-40); draw_bode_mag_tick(-60); 

  glBegin(GL_LINES);
  glVertex2f(AXIS_OFFSET, BODE_PH_ORIGIN + BODE_PH_SCALE*(10.0));
  glVertex2f(AXIS_OFFSET, BODE_PH_ORIGIN + BODE_PH_SCALE*(-270.0));
  glVertex2f(AXIS_OFFSET, BODE_PH_ORIGIN + BODE_PH_SCALE*(0.0));
  glVertex2f(AXIS_OFFSET+300, BODE_PH_ORIGIN + BODE_PH_SCALE*(0.0));
  glVertex2f(AXIS_OFFSET, BODE_PH_ORIGIN + BODE_PH_SCALE*(-180.0));
  glVertex2f(AXIS_OFFSET+300, BODE_PH_ORIGIN + BODE_PH_SCALE*(-180.0));
  glEnd();
  draw_bode_ph_tick(0);  draw_bode_ph_tick(-180);draw_bode_ph_tick(-90);draw_bode_ph_tick(-270);
  

  if(controller){
  glLineStipple(1, stipple);
  glEnable(GL_LINE_STIPPLE);
  }
  // Draw the historical Bode trace
  kpdb=20.0*log10(kp);
  glColor3f(1.0, 0.0, 1.0);
  glBegin(GL_LINE_STRIP);
  for (i=50; i<151; i++) {
    glVertex2f(AXIS_OFFSET + (i-50)*3, BODE_MAG_ORIGIN + BODE_MAG_SCALE*(kpdb+frsp_mag[i]));
  }
  glEnd();

  if(controller)
   glDisable(GL_LINE_STIPPLE);

if(controller){  // Draw the historical Bode trace
  kpdb=20.0*log10(kp);
  glColor3f(1.0, 0.0, 1.0);
  glBegin(GL_LINE_STRIP);
  for (i=50; i<151; i++) {
    glVertex2f(AXIS_OFFSET + (i-50)*3, BODE_MAG_ORIGIN + BODE_MAG_SCALE*(kpdb+frsp_mag[i]+kfrsp_mag[i]));
  }
  glEnd();
}

  if(sinusoid){
    glColor3f(0.0, 1.0, 1.0);
    glBegin(GL_POLYGON);
    glVertex2f(AXIS_OFFSET + 150+log10(omega)*150.0+3, BODE_MAG_ORIGIN + BODE_MAG_SCALE*20*log10(kp*abs(frspw))+3);
    glVertex2f(AXIS_OFFSET + 150+log10(omega)*150.0-3, BODE_MAG_ORIGIN + BODE_MAG_SCALE*20*log10(kp*abs(frspw))+3);
    glVertex2f(AXIS_OFFSET + 150+log10(omega)*150.0-3, BODE_MAG_ORIGIN + BODE_MAG_SCALE*20*log10(kp*abs(frspw))-3);
    glVertex2f(AXIS_OFFSET + 150+log10(omega)*150.0+3, BODE_MAG_ORIGIN + BODE_MAG_SCALE*20*log10(kp*abs(frspw))-3);
    glEnd();
  } 

 if(controller){
  glLineStipple(1, stipple);
  glEnable(GL_LINE_STIPPLE);
  }
   // Draw the historical Bode trace
  glColor3f(0.0, 1.0, 1.0);
  glBegin(GL_LINE_STRIP);
  for (i=50; i<151; i++) {
    glVertex2f(AXIS_OFFSET + (i-50)*3, BODE_PH_ORIGIN + BODE_PH_SCALE*frsp_ph[i]);
  }
  glEnd();
  if(controller)
   glDisable(GL_LINE_STIPPLE);
 
 if (controller){
  // Draw the historical Bode trace
  glColor3f(0.0, 1.0, 1.0);
  glBegin(GL_LINE_STRIP);
  for (i=50; i<151; i++) {
    glVertex2f(AXIS_OFFSET + (i-50)*3, BODE_PH_ORIGIN + BODE_PH_SCALE*(frsp_ph[i]+kfrsp_ph[i]));
  }
  glEnd();


  }

  if(sinusoid){
    ph=arg(frspw)*180.0/3.1415927;
    if (ph>0) ph=ph-360;
    glColor3f(0.0, 1.0, 1.0);
    glBegin(GL_POLYGON);
    glVertex2f(AXIS_OFFSET + 150+log10(omega)*150.0+3, BODE_PH_ORIGIN + BODE_PH_SCALE*ph+3);
    glVertex2f(AXIS_OFFSET + 150+log10(omega)*150.0+3, BODE_PH_ORIGIN + BODE_PH_SCALE*ph-3);
    glVertex2f(AXIS_OFFSET + 150+log10(omega)*150.0-3, BODE_PH_ORIGIN + BODE_PH_SCALE*ph-3);
    glVertex2f(AXIS_OFFSET + 150+log10(omega)*150.0-3, BODE_PH_ORIGIN + BODE_PH_SCALE*ph+3);
    glEnd();
  }

  glutSwapBuffers();
}

void draw_cont_window (void)
  // Draws the C_Bode graph
{
  int i, j;
  float ph;
  complex<float> z;
  complex<float> jomegat;
  GLushort old_stipple;
  static GLushort stipple = 0xFF00;

  glutSetWindow(cont_window);
  glClear(GL_COLOR_BUFFER_BIT);

 if (controller){

  b_con=(om_max*.05-sqrt(klf))/(om_max*.05+sqrt(klf));
  d_con=((1.0+b_con)*klf-(1.0-b_con))/2.0;  
  c_con=1.0-b_con+d_con;

  for (i=0; i<151; i++) 
    {
    if(i>0){
      jomegat=pow(10.0,(i-100.0)/50.0)*delta_t;
      jomegat=jomegat*sqrt((complex<float>)-1);
      z=exp(jomegat);
    }
    else z=1;
    kfrsp[i]=((complex<float>)c_con*z+(complex<float>)d_con)/(z+(complex<float>)b_con);
    kfrsp_ph[i]=arg(kfrsp[i])*180.0/3.1415927;
    kfrsp_mag[i]=20.0*log10(abs(kfrsp[i]));
    }

  numc[0]=c_con;
  numc[1]=d_con;
  denc[0]=1.0;
  denc[1]=b_con;

  numl[0]=num[0]*numc[0];
  numl[1]=num[0]*numc[1]+num[1]*numc[0];
  numl[2]=num[1]*numc[1]+num[2]*numc[0];
  numl[3]=num[2]*numc[1];

  denl[0]=den[0]*denc[0];
  denl[1]=den[0]*denc[1]+den[1]*denc[0];
  denl[2]=den[1]*denc[1]+den[2]*denc[0];
  denl[3]=den[2]*denc[1]+den[3]*denc[0];
  denl[4]=den[3]*denc[1] ;

  glutSetWindow(cont_window);
  glClear(GL_COLOR_BUFFER_BIT);


  // Draw axis, label and ticks
  glColor3f(1.0, 1.0, 1.0);
  glBegin(GL_LINES);
  glVertex2f(AXIS_OFFSET, C_BODE_MAG_ORIGIN + C_BODE_MAG_SCALE*(-40.0));
  glVertex2f(AXIS_OFFSET, C_BODE_MAG_ORIGIN + C_BODE_MAG_SCALE*(40.0));
  glVertex2f(AXIS_OFFSET, C_BODE_MAG_ORIGIN + C_BODE_MAG_SCALE*(0.0));
  glVertex2f(AXIS_OFFSET+300, C_BODE_MAG_ORIGIN + C_BODE_MAG_SCALE*(0.0));
  glEnd();
  glut_print(AXIS_OFFSET+25,C_BODE_MAG_ORIGIN + C_BODE_MAG_SCALE*(40.0)+15,"Compensator Bode diagram K(jw)");
  glut_print(AXIS_OFFSET+5,C_BODE_MAG_ORIGIN + C_BODE_MAG_SCALE*(-40.0)-5,"Click and drag (up/down) on the gain plot to change");
  glut_print(AXIS_OFFSET+5,C_BODE_MAG_ORIGIN + C_BODE_MAG_SCALE*(-40.0)-20,"  the low frequency gain (>0dB=lag/<0dB=lead)");
  glut_print(AXIS_OFFSET-25.0,C_BODE_MAG_ORIGIN + C_BODE_MAG_SCALE*(40.0)+15,"dB");
  draw_c_bode_mag_tick(0);   draw_c_bode_mag_tick(-20); 
  draw_c_bode_mag_tick(20); draw_c_bode_mag_tick(-40); draw_c_bode_mag_tick(40); 


  glBegin(GL_LINES);
  glVertex2f(AXIS_OFFSET, C_BODE_PH_ORIGIN + C_BODE_PH_SCALE*(100.0));
  glVertex2f(AXIS_OFFSET, C_BODE_PH_ORIGIN + C_BODE_PH_SCALE*(-100.0));
  glVertex2f(AXIS_OFFSET, C_BODE_PH_ORIGIN + C_BODE_PH_SCALE*(0.0));
  glVertex2f(AXIS_OFFSET+300, C_BODE_PH_ORIGIN + C_BODE_PH_SCALE*(0.0));
  glEnd();
  draw_c_bode_ph_tick(0);draw_c_bode_ph_tick(-90);draw_c_bode_ph_tick(90);
  glut_print(AXIS_OFFSET+10,C_BODE_PH_ORIGIN + C_BODE_PH_SCALE*(-90.0)-5,"Click and drag (left/right) on the phase plot to");
  glut_print(AXIS_OFFSET+10,C_BODE_PH_ORIGIN + C_BODE_PH_SCALE*(-90.0)-20,"  change the centre frequency");
  

  // Draw the historical C_Bode trace
  glColor3f(1.0, 0.0, 1.0);
  glBegin(GL_LINE_STRIP);
  for (i=50; i<151; i++) {
    glVertex2f(AXIS_OFFSET + (i-50)*3, C_BODE_MAG_ORIGIN + C_BODE_MAG_SCALE*(kfrsp_mag[i]));
  }
  glEnd();

  if(sinusoid){
    jomegat=omega*delta_t;
    jomegat=jomegat*sqrt((complex<float>)-1);
    z=exp(jomegat);
    kfrspw=((complex<float>)c_con*z+(complex<float>)d_con)/(z+(complex<float>)b_con);
    glColor3f(0.0, 1.0, 1.0);
    glBegin(GL_POLYGON);
    glVertex2f(AXIS_OFFSET + 150+log10(omega)*150.0+3, C_BODE_MAG_ORIGIN + C_BODE_MAG_SCALE*20*log10(kp*abs(kfrspw))+3);
    glVertex2f(AXIS_OFFSET + 150+log10(omega)*150.0-3, C_BODE_MAG_ORIGIN + C_BODE_MAG_SCALE*20*log10(kp*abs(kfrspw))+3);
    glVertex2f(AXIS_OFFSET + 150+log10(omega)*150.0-3, C_BODE_MAG_ORIGIN + C_BODE_MAG_SCALE*20*log10(kp*abs(kfrspw))-3);
    glVertex2f(AXIS_OFFSET + 150+log10(omega)*150.0+3, C_BODE_MAG_ORIGIN + C_BODE_MAG_SCALE*20*log10(kp*abs(kfrspw))-3);
    glEnd();
  } 

  // Draw the historical C_Bode trace
  glColor3f(1.0, 0.0, 1.0);
  glBegin(GL_LINE_STRIP);
  for (i=50; i<151; i++) {
    glVertex2f(AXIS_OFFSET + (i-50)*3, C_BODE_PH_ORIGIN + C_BODE_PH_SCALE*kfrsp_ph[i]);
  }
  glEnd();
  if(sinusoid){
    ph=arg(kfrspw)*180.0/3.1415927;
    glColor3f(0.0, 1.0, 1.0);
    glBegin(GL_POLYGON);
    glVertex2f(AXIS_OFFSET + 150+log10(omega)*150.0+3, C_BODE_PH_ORIGIN + C_BODE_PH_SCALE*ph+3);
    glVertex2f(AXIS_OFFSET + 150+log10(omega)*150.0+3, C_BODE_PH_ORIGIN + C_BODE_PH_SCALE*ph-3);
    glVertex2f(AXIS_OFFSET + 150+log10(omega)*150.0-3, C_BODE_PH_ORIGIN + C_BODE_PH_SCALE*ph-3);
    glVertex2f(AXIS_OFFSET + 150+log10(omega)*150.0-3, C_BODE_PH_ORIGIN + C_BODE_PH_SCALE*ph+3);
    glEnd();
  }
  }
  glutSwapBuffers();
  
}

void draw_inst_window (void)
{
 glutSetWindow(inst_window);
  glClear(GL_COLOR_BUFFER_BIT);
  glut_print(0,subwin_height-10,"Instructions:");
  glut_print(0,subwin_height-30,"Anything in");
    glColor3f(1.0, 0.0, 1.0);
  glut_print(70,subwin_height-30,"magenta");
    glColor3f(1.0, 1.0, 1.0);
  glut_print(125,subwin_height-30,"is \"click and dragable\".");
 glut_print(0,subwin_height-50,"Click and \"wiggle\" in the middle left window to set a");
 glut_print(0,subwin_height-65,"  reference.");
 glut_print(0,subwin_height-85,"Press \"s\" to toggle on/off a sinusoidal reference signal.");

 glut_print(0,subwin_height-105,"Click and drag (up and down) on the magenta numbers");
 glut_print(0,subwin_height-120,"  in the top left figure to change the gain and the");
glut_print(0,subwin_height-135,"  value or frequency of the reference signal.");  
 glut_print(0,subwin_height-155,"Press the space bar to pause the scrolling traces.");
 glut_print(0,subwin_height-175,"Press z to zero the scrolling traces.");
 glut_print(0,subwin_height-195,"Use the cursor keys to adjust the speed of the scrolling");
 glut_print(0,subwin_height-210,"traces.");
 glut_print(0,subwin_height-225,"Press c to add a phase compensator.");
 glut_print(0,subwin_height-240,"Press q to quit.");

 glutSwapBuffers();
}

void draw_clfr_window (void)
  // Draws the Bode graph
{
  int i, j;
  GLushort old_stipple;
  static GLushort stipple = 0xFF00;

  glutSetWindow(clfr_window);
  glClear(GL_COLOR_BUFFER_BIT);

  // Draw axis, label and ticks
  glColor3f(1.0, 1.0, 1.0);
  glBegin(GL_LINES);
  glVertex2f(AXIS_OFFSET, CLFR_ORIGIN + CLFR_SCALE*(-60.0));
  glVertex2f(AXIS_OFFSET, CLFR_ORIGIN + CLFR_SCALE*(20.0));
  glVertex2f(AXIS_OFFSET, CLFR_ORIGIN + CLFR_SCALE*(0.0));
  glVertex2f(AXIS_OFFSET+300, CLFR_ORIGIN + CLFR_SCALE*(0.0));
  glEnd();
  glut_print(AXIS_OFFSET-25.0,CLFR_ORIGIN + CLFR_SCALE*(20.0)+15,"dB");
  glut_print(AXIS_OFFSET-25.0,CLFR_ORIGIN + CLFR_SCALE*(20.0)+40,"Closed-loop frequency responses:");
  draw_clfr_tick(0);   draw_clfr_tick(-20); 
  draw_clfr_tick(20); draw_clfr_tick(-40); draw_clfr_tick(-60); 



  // Draw the historical Bode trace
  if (kp<kcrit){
  glColor3f(1.0, 1.0, 0.0);
  glBegin(GL_LINE_STRIP);
  for (i=50; i<151; i++) {
if (controller)
    glVertex2f(AXIS_OFFSET + (i-50)*3, CLFR_ORIGIN + CLFR_SCALE*20*log10(abs((complex<float>)1.0/((complex<float>)1.0+kp*(frsp[i]*kfrsp[i])))));
 else
    glVertex2f(AXIS_OFFSET + (i-50)*3, CLFR_ORIGIN + CLFR_SCALE*20*log10(abs((complex<float>)1.0/((complex<float>)1.0+kp*(frsp[i])))));
  }
  glEnd();
  glut_print(AXIS_OFFSET+25.0,CLFR_ORIGIN + CLFR_SCALE*(20.0)+25,"Sensitivity: [ error(jw) / ref(jw) ]");

  glColor3f(0.0, 1.0, 1.0);
 glBegin(GL_LINE_STRIP);
  for (i=50; i<151; i++) {
if (controller)
    glVertex2f(AXIS_OFFSET + (i-50)*3, CLFR_ORIGIN + CLFR_SCALE*20*log10(abs((complex<float>)1.0-(complex<float>)1.0/((complex<float>)1.0+kp*(frsp[i]*kfrsp[i])))));
 else
    glVertex2f(AXIS_OFFSET + (i-50)*3, CLFR_ORIGIN + CLFR_SCALE*20*log10(abs((complex<float>)1.0-(complex<float>)1.0/((complex<float>)1.0+kp*(frsp[i])))));
  }
  glEnd();
  glut_print(AXIS_OFFSET+25.0,CLFR_ORIGIN + CLFR_SCALE*(20.0)+10,"Complementary Sensitivity: [ y(jw) / ref(jw) ]");
  }
  if(sinusoid && (kp<kcrit) ){
    glColor3f(0.0, 1.0, 1.0);
    glBegin(GL_POLYGON);
    glVertex2f(AXIS_OFFSET + 150+log10(omega)*150.0+3, CLFR_ORIGIN + CLFR_SCALE*20*log10(abs((complex<float>)1.0-(complex<float>)1.0/((complex<float>)1.0+kp*(frspw))))+3);
    glVertex2f(AXIS_OFFSET + 150+log10(omega)*150.0-3, CLFR_ORIGIN + CLFR_SCALE*20*log10(abs((complex<float>)1.0-(complex<float>)1.0/((complex<float>)1.0+kp*(frspw))))+3);
    glVertex2f(AXIS_OFFSET + 150+log10(omega)*150.0-3, CLFR_ORIGIN + CLFR_SCALE*20*log10(abs((complex<float>)1.0-(complex<float>)1.0/((complex<float>)1.0+kp*(frspw))))-3);
    glVertex2f(AXIS_OFFSET + 150+log10(omega)*150.0+3, CLFR_ORIGIN + CLFR_SCALE*20*log10(abs((complex<float>)1.0-(complex<float>)1.0/((complex<float>)1.0+kp*(frspw))))-3);
    glEnd();
    glBegin(GL_POLYGON);
    glVertex2f(AXIS_OFFSET + 150+log10(omega)*150.0+3, CLFR_ORIGIN + CLFR_SCALE*20*log10(abs((complex<float>)1.0/((complex<float>)1.0+kp*(frspw))))+3);
    glVertex2f(AXIS_OFFSET + 150+log10(omega)*150.0-3, CLFR_ORIGIN + CLFR_SCALE*20*log10(abs((complex<float>)1.0/((complex<float>)1.0+kp*(frspw))))+3);
    glVertex2f(AXIS_OFFSET + 150+log10(omega)*150.0-3, CLFR_ORIGIN + CLFR_SCALE*20*log10(abs((complex<float>)1.0/((complex<float>)1.0+kp*(frspw))))-3);
    glVertex2f(AXIS_OFFSET + 150+log10(omega)*150.0+3, CLFR_ORIGIN + CLFR_SCALE*20*log10(abs((complex<float>)1.0/((complex<float>)1.0+kp*(frspw))))-3);
    glEnd();
  }
  
  glutSwapBuffers();
} 

void draw_nyquist_window (void)
  // Draws the Nyquist graph
{
  int i, j;
  GLushort old_stipple;
  static GLushort stipple = 0xFF00;

  glutSetWindow(nyquist_window);
  glClear(GL_COLOR_BUFFER_BIT);

  // Draw axis, label and ticks
  glColor3f(1.0, 1.0, 1.0);
  glBegin(GL_LINES);
  glVertex2f(NYQUIST_X_ORIGIN, NYQUIST_Y_ORIGIN + NYQUIST_SCALE*(-1.5));
  glVertex2f(NYQUIST_X_ORIGIN, NYQUIST_Y_ORIGIN + NYQUIST_SCALE*(1.5));
  glVertex2f(NYQUIST_X_ORIGIN, NYQUIST_Y_ORIGIN + NYQUIST_SCALE*(1.5));
  glVertex2f(NYQUIST_X_ORIGIN-3, NYQUIST_Y_ORIGIN + NYQUIST_SCALE*(1.5)-7);
  glVertex2f(NYQUIST_X_ORIGIN, NYQUIST_Y_ORIGIN + NYQUIST_SCALE*(1.5));
  glVertex2f(NYQUIST_X_ORIGIN+3, NYQUIST_Y_ORIGIN + NYQUIST_SCALE*(1.5)-7);
  glVertex2f(NYQUIST_X_ORIGIN + NYQUIST_SCALE*(-1.0), NYQUIST_Y_ORIGIN + NYQUIST_SCALE*(-.1));
  glVertex2f(NYQUIST_X_ORIGIN + NYQUIST_SCALE*(-1.0), NYQUIST_Y_ORIGIN + NYQUIST_SCALE*(.1));
  glVertex2f(NYQUIST_X_ORIGIN + NYQUIST_SCALE*(-1.5), NYQUIST_Y_ORIGIN);
  glVertex2f(NYQUIST_X_ORIGIN + NYQUIST_SCALE*(1.5), NYQUIST_Y_ORIGIN);
  glVertex2f(NYQUIST_X_ORIGIN + NYQUIST_SCALE*(1.5), NYQUIST_Y_ORIGIN);
  glVertex2f(NYQUIST_X_ORIGIN + NYQUIST_SCALE*(1.5)-7, NYQUIST_Y_ORIGIN+3);
  glVertex2f(NYQUIST_X_ORIGIN + NYQUIST_SCALE*(1.5), NYQUIST_Y_ORIGIN);
  glVertex2f(NYQUIST_X_ORIGIN + NYQUIST_SCALE*(1.5)-7, NYQUIST_Y_ORIGIN-3);
  glEnd();
  glut_print(NYQUIST_X_ORIGIN,NYQUIST_Y_ORIGIN + NYQUIST_SCALE*(1.55),"Imag(G(jw))");
  glut_print(NYQUIST_X_ORIGIN+ NYQUIST_SCALE*(1.25),NYQUIST_Y_ORIGIN -20,"Real(G(jw))");
  glut_print(NYQUIST_X_ORIGIN+ NYQUIST_SCALE*(-1.0)-5,NYQUIST_Y_ORIGIN +10,"-1");
  glut_print(AXIS_OFFSET-25.0,NYQUIST_Y_ORIGIN + NYQUIST_SCALE*(1.4),"Nyquist diagram:");
  glut_print(AXIS_OFFSET-25.0,NYQUIST_Y_ORIGIN + NYQUIST_SCALE*(1.4)-15,"  [ y(jw) / error(jw) ]");
  //  draw_nyquist_tick(0); draw_nyquist_tick(-5);draw_nyquist_tick(-10); 
  // draw_nyquist_tick(10); draw_nyquist_tick(5);


  if(controller){
  glLineStipple(1, stipple);
  glEnable(GL_LINE_STIPPLE);
  }
  // Draw the Nyquist locus
  glColor3f(1.0, 0.0, 1.0);
  glBegin(GL_LINE_STRIP);
    for (i=0; i<131; i++) {
    glVertex2f(NYQUIST_X_ORIGIN + NYQUIST_SCALE*kp*frsp_real[i], NYQUIST_Y_ORIGIN + NYQUIST_SCALE*kp*frsp_imag[i]);
    }
  glEnd();
  if(controller)
   glDisable(GL_LINE_STIPPLE);

  if(controller)
    {
      // Draw the Nyquist locus
      glColor3f(1.0, 0.0, 1.0);
      glBegin(GL_LINE_STRIP);
      for (i=0; i<131; i++) {
	glVertex2f(NYQUIST_X_ORIGIN + NYQUIST_SCALE*kp*real(frsp[i]*kfrsp[i]), NYQUIST_Y_ORIGIN + NYQUIST_SCALE*kp*imag(frsp[i]*kfrsp[i]));
      }
      glEnd();
    }

  if(sinusoid){  
    glColor3f(0.0, 1.0, 1.0);
    glBegin(GL_POLYGON);
    glVertex2f(NYQUIST_X_ORIGIN + NYQUIST_SCALE*kp*real(frspw)-3, NYQUIST_Y_ORIGIN + NYQUIST_SCALE*kp*imag(frspw)-3);
    glVertex2f(NYQUIST_X_ORIGIN + NYQUIST_SCALE*kp*real(frspw)-3, NYQUIST_Y_ORIGIN + NYQUIST_SCALE*kp*imag(frspw)+3);
    glVertex2f(NYQUIST_X_ORIGIN + NYQUIST_SCALE*kp*real(frspw)+3, NYQUIST_Y_ORIGIN + NYQUIST_SCALE*kp*imag(frspw)+3);
    glVertex2f(NYQUIST_X_ORIGIN + NYQUIST_SCALE*kp*real(frspw)+3, NYQUIST_Y_ORIGIN + NYQUIST_SCALE*kp*imag(frspw)-3);
    glEnd();
  }
  glutSwapBuffers();
}



void draw_circuit_window (void)
  // Draws the circuit diagram
{
  char mess[12];
  float a;

  glutSetWindow(circuit_window);
  glClear(GL_COLOR_BUFFER_BIT);
  glColor3f(1.0, 1.0, 1.0);

  // G 
  glBegin(GL_LINE_LOOP);
  glVertex2f(G_X_POS+G_X_SIZE,G_Y_POS+G_Y_SIZE);
  glVertex2f(G_X_POS+G_X_SIZE,G_Y_POS-G_Y_SIZE);
  glVertex2f(G_X_POS-G_X_SIZE,G_Y_POS-G_Y_SIZE);
  glVertex2f(G_X_POS-G_X_SIZE,G_Y_POS+G_Y_SIZE);
  glEnd();
  sprintf(mess,"G(s)");
  glut_print(G_X_POS-10, G_Y_POS-5, mess);
  // K
  glBegin(GL_LINE_LOOP);
  glVertex2f(K_X_POS+G_X_SIZE,G_Y_POS+G_Y_SIZE);
  glVertex2f(K_X_POS+G_X_SIZE,G_Y_POS-G_Y_SIZE);
  glVertex2f(K_X_POS-G_X_SIZE,G_Y_POS-G_Y_SIZE);
  glVertex2f(K_X_POS-G_X_SIZE,G_Y_POS+G_Y_SIZE);
  glEnd();
if (controller)  glut_print(K_X_POS-G_X_SIZE+5, G_Y_POS-5, "kp K(s)");
else  glut_print(K_X_POS-G_X_SIZE+20, G_Y_POS-5, "kp");
  // G to K line
  glBegin(GL_LINES);
  glVertex2f(K_X_POS+G_X_SIZE,G_Y_POS);
  glVertex2f(G_X_POS-G_X_SIZE,G_Y_POS);
  glVertex2f(G_X_POS-G_X_SIZE,G_Y_POS);
  glVertex2f(G_X_POS-G_X_SIZE-7,G_Y_POS+3);
  glVertex2f(G_X_POS-G_X_SIZE,G_Y_POS);
  glVertex2f(G_X_POS-G_X_SIZE-7,G_Y_POS-3);
  glEnd();
  

  // G to sum jct loop
  glBegin(GL_LINE_STRIP);
  glVertex2f(G_X_POS+G_X_SIZE,G_Y_POS);
  glVertex2f(G_X_POS+G_X_SIZE+10,G_Y_POS);
  glVertex2f(G_X_POS+G_X_SIZE+10,G_Y_POS-Y_LOOP);
  glVertex2f(S_X_POS,G_Y_POS-Y_LOOP);
  glVertex2f(S_X_POS,G_Y_POS-10);
  glVertex2f(S_X_POS+3,G_Y_POS-10-7);
  glVertex2f(S_X_POS,G_Y_POS-10);
  glVertex2f(S_X_POS-3,G_Y_POS-10-7);
  glEnd();
  sprintf(mess,"-");
  glut_print(S_X_POS-12, G_Y_POS-20, mess);


  // G to out
  glBegin(GL_LINE_STRIP);
  glVertex2f(G_X_POS+G_X_SIZE,G_Y_POS);
  glVertex2f(G_X_POS+G_X_SIZE+30,G_Y_POS);
  glVertex2f(G_X_POS+G_X_SIZE+30-7,G_Y_POS+3);
  glVertex2f(G_X_POS+G_X_SIZE+30,G_Y_POS);
  glVertex2f(G_X_POS+G_X_SIZE+30-7,G_Y_POS-3);
  glEnd();
  glColor3f(0.0, 1.0, 1.0);  
  sprintf(mess,"y");
  glut_print(G_X_POS+G_X_SIZE+15, G_Y_POS+5, mess);
  glColor3f(1.0, 1.0, 1.0);  

  // Sum jct
  glBegin(GL_LINE_LOOP);
  for (a=0; a<2*M_PI; a+=0.1) {
    glVertex2f(S_X_POS + 10.0*cos(a), G_Y_POS + 10.0*sin(a));
  }
  glEnd();

  // Sum to K line
  glBegin(GL_LINES);
  glVertex2f(S_X_POS+10,G_Y_POS);
  glVertex2f(K_X_POS-G_X_SIZE,G_Y_POS);
  glVertex2f(K_X_POS-G_X_SIZE,G_Y_POS);
  glVertex2f(K_X_POS-G_X_SIZE-7,G_Y_POS+3);
  glVertex2f(K_X_POS-G_X_SIZE,G_Y_POS);
  glVertex2f(K_X_POS-G_X_SIZE-7,G_Y_POS-3);
  glEnd();

  // in to sum line
  glBegin(GL_LINES);
  glVertex2f(S_X_POS-40,G_Y_POS);
  glVertex2f(S_X_POS-10,G_Y_POS);
  glVertex2f(S_X_POS-10,G_Y_POS);
  glVertex2f(S_X_POS-10-7,G_Y_POS+3);
  glVertex2f(S_X_POS-10,G_Y_POS);
  glVertex2f(S_X_POS-10-7,G_Y_POS-3);
  glEnd();


    sprintf(mess,"error");
    glColor3f(1.0, 1.0, 0.0);
    glut_print(S_X_POS+12, G_Y_POS+5, mess);

  // Adjustable parameters in magenta
  
  if (sinusoid) {
    sprintf(mess,"omega=%3.2f", omega);
    glColor3f(1.0, 0.0, 1.0);
    glut_print(OM_X_POS-G_X_SIZE+5, G_Y_POS-15, mess);
    sprintf(mess,"ref=cos(wt)");
    glColor3f(1.0, 0.0, 1.0);
    glut_print(OM_X_POS-G_X_SIZE+5, G_Y_POS+5, mess);}
  else {sprintf(mess,"ref=%3.2f", signal_voltage);
    glColor3f(1.0, 0.0, 1.0);
    glut_print(OM_X_POS-G_X_SIZE+25, G_Y_POS+5, mess);}
  
  glColor3f(1.0, 0.0, 1.0);
  sprintf(mess,"kp=%3.2f", kp);
  glut_print(K_X_POS-G_X_SIZE+5, G_Y_POS-30, mess);


  

  glutSwapBuffers();
}

void draw_characteristic_window (void)
  // Draws the characteristic graph
{
  int i;
  static int nroots;
  double inpoly[5], zerosr[5], zerosi[5];

  float  a, b, c, d, e;
  static complex<double> zs[5];
  complex<float> z;
  complex<float> jomegat;
  static float kp_last;
  static float klf_last;
  static float om_max_last;
  static bool controller_last;
  glutSetWindow(characteristic_window);
  glClear(GL_COLOR_BUFFER_BIT);
  // Draw axis, label and ticks
  glColor3f(1.0, 1.0, 1.0);
  glBegin(GL_LINES);
  glVertex2f(RLOCUS_X_ORIGIN, RLOCUS_Y_ORIGIN + RLOCUS_SCALE*(-2));
  glVertex2f(RLOCUS_X_ORIGIN, RLOCUS_Y_ORIGIN + RLOCUS_SCALE*(2));
  glVertex2f(RLOCUS_X_ORIGIN + RLOCUS_SCALE*(-2), RLOCUS_Y_ORIGIN);
  glVertex2f(RLOCUS_X_ORIGIN + RLOCUS_SCALE*(2), RLOCUS_Y_ORIGIN);
  glVertex2f(RLOCUS_X_ORIGIN, RLOCUS_Y_ORIGIN + RLOCUS_SCALE*(2));
  glVertex2f(RLOCUS_X_ORIGIN-3, RLOCUS_Y_ORIGIN + RLOCUS_SCALE*(2)-7);
  glVertex2f(RLOCUS_X_ORIGIN, RLOCUS_Y_ORIGIN + RLOCUS_SCALE*(2));
  glVertex2f(RLOCUS_X_ORIGIN+3, RLOCUS_Y_ORIGIN + RLOCUS_SCALE*(2)-7);
 glVertex2f(RLOCUS_X_ORIGIN + RLOCUS_SCALE*(2), RLOCUS_Y_ORIGIN);
  glVertex2f(RLOCUS_X_ORIGIN + RLOCUS_SCALE*(2)-7, RLOCUS_Y_ORIGIN+3);
  glVertex2f(RLOCUS_X_ORIGIN + RLOCUS_SCALE*(2), RLOCUS_Y_ORIGIN);
  glVertex2f(RLOCUS_X_ORIGIN + RLOCUS_SCALE*(2)-7, RLOCUS_Y_ORIGIN-3);
 glEnd();
  glut_print(AXIS_OFFSET-25.0,RLOCUS_Y_ORIGIN + RLOCUS_SCALE*(1.6),"Closed-loop poles");
  glut_print(RLOCUS_X_ORIGIN+10,RLOCUS_Y_ORIGIN + RLOCUS_SCALE*(2)-5,"Imag(s)");
  glut_print(RLOCUS_X_ORIGIN+ RLOCUS_SCALE*(2)-20,RLOCUS_Y_ORIGIN -20,"Real(s)");
  //  draw_nyquist_tick(0); draw_nyquist_tick(-5);draw_nyquist_tick(-10); 
  // draw_nyquist_tick(10); draw_nyquist_tick(5);

  if (!controller)
    {
      if (kp!=kp_last || controller_last)
	{
	  a=den[0];
	  b=den[1]+kp*num[0];
	  c=den[2]+kp*num[1];
	  d=den[3]+kp*num[2];

	  inpoly[0]=a;
	  inpoly[1]=b;
	  inpoly[2]=c;
	  inpoly[3]=d;
	  
	  nroots=rpoly(inpoly, 3, zerosr, zerosi);
	  
	  for (i=0;i<nroots;i++)
	    {
	      zs[i]=(complex<double>)zerosr[i]+(complex<double>)zerosi[i]*sqrt(complex<double>(-1));
	      zs[i]=log(zs[i])/(complex<double>)delta_t;
	    }
        }
    }
  else      
    {
      if (kp != kp_last || klf != klf_last || om_max_last != om_max || !controller_last)
	{
	  a=denl[0];
	  b=denl[1]+kp*numl[0];
	  c=denl[2]+kp*numl[1];
	  d=denl[3]+kp*numl[2];
	  e=denl[4]+kp*numl[3];

	  inpoly[0]=a;
	  inpoly[1]=b;
	  inpoly[2]=c;
	  inpoly[3]=d;
	  inpoly[4]=e;
	  
	  nroots=rpoly(inpoly, 4, zerosr, zerosi);

	  for (i=0;i<nroots;i++)
	    {
	      zs[i]=(complex<double>)zerosr[i]+(complex<double>)zerosi[i]*sqrt(complex<double>(-1));
	      zs[i]=log(zs[i])/(complex<double>)delta_t;
	    }
	  kp_last=kp;
        }
    }
  kp_last=kp;
  klf_last=klf;
  om_max_last=om_max;
  controller_last=controller;
  
  jomegat=omega*delta_t;
  jomegat=jomegat*sqrt((complex<float>)-1);
  z=exp(jomegat);
  if (!controller)
  frspw= ((complex<float>)num[0]*pow(z,2)+(complex<float>)num[1]*pow(z,1)+(complex<float>)num[2])/((complex<float>)den[0]*pow(z,3)+(complex<float>)den[1]*pow(z,2)+(complex<float>)den[2]*pow(z,1)+(complex<float>)den[3]); 
  else
  frspw= ((complex<float>)numl[0]*pow(z,3)+(complex<float>)numl[1]*pow(z,2)+(complex<float>)numl[2]*z+(complex<float>)numl[3])/((complex<float>)denl[0]*pow(z,4)+(complex<float>)denl[1]*pow(z,3)+(complex<float>)denl[2]*pow(z,2)+(complex<float>)denl[3]*z+(complex<float>)denl[4]); 
 
  glColor3f(0.0, 1.0, 1.0);
  glBegin(GL_LINES);
  for (i=0;i<nroots;i++)
    {
      glVertex2f(NYQUIST_X_ORIGIN + NYQUIST_SCALE*(real(zs[i])), NYQUIST_Y_ORIGIN + NYQUIST_SCALE*(imag(zs[i])-.1));
      glVertex2f(NYQUIST_X_ORIGIN + NYQUIST_SCALE*(real(zs[i])), NYQUIST_Y_ORIGIN + NYQUIST_SCALE*(imag(zs[i])+.1));
      glVertex2f(NYQUIST_X_ORIGIN + NYQUIST_SCALE*(real(zs[i])-.1), NYQUIST_Y_ORIGIN + NYQUIST_SCALE*(imag(zs[i])));
      glVertex2f(NYQUIST_X_ORIGIN + NYQUIST_SCALE*(real(zs[i])+.1), NYQUIST_Y_ORIGIN + NYQUIST_SCALE*(imag(zs[i])));
    }

 glEnd();

 if(sinusoid){
   glColor3f(0.0, 1.0, 1.0);
   glBegin(GL_POLYGON);
   glVertex2f(NYQUIST_X_ORIGIN + NYQUIST_SCALE*(0.0)-3, NYQUIST_Y_ORIGIN + NYQUIST_SCALE*(omega)-3);
   glVertex2f(NYQUIST_X_ORIGIN + NYQUIST_SCALE*(0.0)+3, NYQUIST_Y_ORIGIN + NYQUIST_SCALE*(omega)-3);
   glVertex2f(NYQUIST_X_ORIGIN + NYQUIST_SCALE*(0.0)+3, NYQUIST_Y_ORIGIN + NYQUIST_SCALE*(omega)+3);
   glVertex2f(NYQUIST_X_ORIGIN + NYQUIST_SCALE*(0.0)-3, NYQUIST_Y_ORIGIN + NYQUIST_SCALE*(omega)+3);
   glEnd();
 }
 glutSwapBuffers(); 
}

void draw_main_window (void)
  // Draw grey lines to partition the display into four sub-windows
{
  glutSetWindow(main_window);
  glClear(GL_COLOR_BUFFER_BIT);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glColor3f(0.5, 0.5, 0.5);
  glBegin(GL_LINES);
  glVertex2i(subwin_width + 2*GAP, 0);
  glVertex2i(subwin_width + 2*GAP, win_height);
  glVertex2i(2*subwin_width + 4*GAP, 0);
  glVertex2i(2*subwin_width + 4*GAP, win_height);
  glVertex2i(0, subwin_height + 2*GAP);
  glVertex2i(win_width, subwin_height + 2*GAP);
  glVertex2i(0, 2*subwin_height + 4*GAP);
  glVertex2i(win_width, 2*subwin_height + 4*GAP);
  glEnd();
  glutSwapBuffers();
}

void refresh_param_subwindows (void)
{
  params_changed=true;
}

void refresh_circuit_subwindow (void)
{
  refresh_circuit=true;
}

void really_refresh_param_subwindows (int i)
{
  if (params_changed)
    {
  //  if (controller){
  glutSetWindow(cont_window);
  glutPostRedisplay();
  //}
    glutSetWindow(nyquist_window);
  glutPostRedisplay();
  glutSetWindow(bode_window);
  glutPostRedisplay();
  glutSetWindow(clfr_window);
  glutPostRedisplay();
  glutSetWindow(circuit_window);
  glutPostRedisplay();
  glutSetWindow(characteristic_window);
  glutPostRedisplay();
  params_changed=false;
    }
  else if (refresh_circuit)
    {
      glutSetWindow(circuit_window);
      glutPostRedisplay();
      refresh_circuit=false;
    }
  glutTimerFunc(40,really_refresh_param_subwindows,0);
}

void draw_live_windows (int i)
  // The idle function, waits a little then updates all subwindows
{
  //  unsigned long long a, b;

  //  microsecond_time(a);
  // do microsecond_time (b); while ((b-a) < microsecond_delay);
  
  tnow+=delta_t;
  theta += omega*delta_t; 

  if (sinusoid) {
    signal_voltage = 5.0*sin(theta);
  }
  glutSetWindow(ref_window);
  glutPostRedisplay();
  glutSetWindow(error_window);
  glutPostRedisplay();
  if (!(paused)) glutTimerFunc(millisecond_delay,draw_live_windows,0);
}

void initialise_subwindow (void)
{
  glutReshapeWindow(subwin_width, subwin_height);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glViewport(0, 0, subwin_width, subwin_height);
  glOrtho(0, subwin_width, 0, subwin_height, -1, 1); 
  glDrawBuffer(GL_BACK); 
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glRasterPos2d(PART_PIXEL, subwin_height - PART_PIXEL);
  glPixelZoom(1.0, -1.0);
  glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
  glutPostRedisplay();
}

void reshape_main_window (int width, int height)
{
  // Resize the main window
  glutSetWindow(main_window);
  win_width = glutGet(GLUT_WINDOW_WIDTH);
  win_height = glutGet(GLUT_WINDOW_HEIGHT);

  subwin_width = (win_width - 6*GAP)/3;
  subwin_height = (win_height - 6*GAP)/3;
  glViewport(0, 0, win_width, win_height);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(0, win_width, 0, win_height, -1, 1);

  // Resize and initialise the characteristic window
  glutSetWindow(characteristic_window);
  glutPositionWindow(2*subwin_width + 5*GAP, 2*subwin_height + 5*GAP);
  initialise_subwindow();
  
  // Resize and initialise the Ref window
  glutSetWindow(ref_window);
  glutPositionWindow(GAP, subwin_height + 3*GAP);
  initialise_subwindow();

  //Resize and initialise the error window
  glutSetWindow(error_window);
  glutPositionWindow(GAP, 2*subwin_height + 5*GAP);
  initialise_subwindow();

  // Resize and initialise the circuit window
  glutSetWindow(circuit_window);
  glutPositionWindow(GAP, GAP);
  initialise_subwindow();

  // Resize and initialise the controller window
  glutSetWindow(cont_window);
  glutPositionWindow(subwin_width + 3*GAP, GAP);
  initialise_subwindow();

  // Resize and initialise the nyquist window
  glutSetWindow(nyquist_window);
  glutPositionWindow(subwin_width + 3*GAP, subwin_height + 3*GAP);
  initialise_subwindow();

  // Resize and initialise the Bode window
  glutSetWindow(bode_window);
  glutPositionWindow(subwin_width + 3*GAP, 2*subwin_height + 5*GAP);
  initialise_subwindow();

  // Resize and initialise the clfr window
  glutSetWindow(clfr_window);
  glutPositionWindow(2*subwin_width + 5*GAP, subwin_height + 3*GAP);
  initialise_subwindow();

  // Resize and initialise the instructions window
  glutSetWindow(inst_window);
  glutPositionWindow(2*subwin_width + 5*GAP, GAP);
  initialise_subwindow();

}

void circuit_mouse_button(int button, int state, int x, int y)
  // Callback for mouse button presses in the circuit window
{
  if (state == GLUT_UP) {
    adjusting_ref = false;
    adjusting_kp = false;
    adjusting_omega = false;
    last_click_y = -1;
  }

  if (state == GLUT_DOWN) {
    // Decide which kp paramater the user is trying to adjust
    if (x<90){
    if (sinusoid) 
      adjusting_omega = true;
    else
      adjusting_ref=true;
    }
    else adjusting_kp=true;
    last_click_y = -1;
  }
}

void circuit_mouse_motion(int x, int y)
  // Callback for mouse drags in the circuit window
{
  if (adjusting_omega) {

    if (last_click_y >= 0) {
      if (y > last_click_y) {
	if ((y-last_click_y)>10) {
	  omega -=.05;
	  last_click_y = y;
	} 
      } else {
	if ((last_click_y-y)>10) {
	  omega+=.05;
	  last_click_y = y;
	} 
      }
      if (omega > 10.0) omega = 10.0;
      if (omega < 0.1) omega = 0.1;
      
    } else last_click_y = y;

  }

  if (adjusting_ref) {

    if (last_click_y >= 0) {
      if (y > last_click_y) {
	if ((y-last_click_y)>1) {
	  signal_voltage-=.05;
	  last_click_y = y;
	} 
      } else {
	if ((last_click_y-y)>1) {
	  signal_voltage+=.05;
	  last_click_y = y;
	} 
      }
      
    } else last_click_y = y;
  }
  
  if (adjusting_kp) {

    if (last_click_y >= 0) {
      if (y > last_click_y) {
	if ((y-last_click_y)>2) {
	  kp -= 0.02;
	  last_click_y = y;
	} 
      } else {
	if ((last_click_y-y)>2) {
	  kp += 0.02;
	  last_click_y = y;
	} 
      }
      if (kp > kp_max) kp = kp_max;
      if (kp < 0.0) kp = 0.0;
    } else last_click_y = y;
  }
  


    refresh_param_subwindows();
  // Make sure the changes are visible, even if paused
}

void ref_mouse_button(int button, int state, int x, int y)
  // Callback for mouse button presses in the Ref window
{
  if (state == GLUT_UP) 
    signal_voltage_orig=signal_voltage;
  last_click_y = -1;
}

void ref_mouse_motion(int x, int y)
  // Callback for mouse drags in the Ref window
{
  if (last_click_y >= 0) {
    if(sinusoid) {
      if ((y-last_click_y)+(last_click_x-x)>2) {
	omega -=.05;
	last_click_y = y;
	last_click_x = x;
      } 
      else {
	if ((last_click_y-y)+(x-last_click_x)>2) {
	  omega+=.05;
	  last_click_x = x;
	  last_click_y = y;
	}
      }
      if (omega > 10.0) omega = 10.0;
      if (omega < 0.1) omega = 0.1;
      refresh_param_subwindows();
     } 
      else {
	signal_voltage = signal_voltage_orig+0.1*(last_click_y-y);  
	if (signal_voltage>10.0) signal_voltage=10.0;
	if (signal_voltage<-10.0) signal_voltage=-10.0;
        refresh_circuit_subwindow();   
      }
  } else last_click_y = y;
}

void nyquist_mouse_button(int button, int state, int x, int y)
  // Callback for mouse button presses in the Nyquist window
{
    last_click_y = -1;
}

void bode_mouse_button(int button, int state, int x, int y)
  // Callback for mouse button presses in the Bode window
{
    last_click_y = -1;
}

void cont_mouse_button(int button, int state, int x, int y)
  // Callback for mouse button presses in the Bode window
{
  if (state == GLUT_UP) {
    adjusting_klf = false;
    adjusting_om_max = false;
  }

  if (state == GLUT_DOWN) {
    // Decide which paramater the user is trying to adjust
    if (controller)
      {
	if (y<140)
	  adjusting_klf=true;
	else
	  adjusting_om_max=true;
      }
  }
  last_click_y = -1;
}

void nyquist_mouse_motion(int x, int y)
  // Callback for mouse drags in the Nyquist window
{
  if (last_click_y >= 0) {
    if ((y-last_click_y) + (x-last_click_x)>2) {
      kp -= 0.05;
      last_click_y = y;
      last_click_x = x;
    } 
    else {
      if ((last_click_y-y)+(last_click_x-x)>2) {
	kp += 0.05;
	last_click_y = y;
	last_click_x = x;
      } 
    }
    if (kp > kp_max) kp = kp_max;
    if (kp < 0.0) kp = 0.0;
  } else last_click_y = y;last_click_x = x;
  refresh_param_subwindows();
}

void bode_mouse_motion(int x, int y)
  // Callback for mouse drags in the Bode window
{
  if (last_click_y >= 0) {
    if ((y-last_click_y)>2) {
      kp /= 1.3;
      last_click_y = y;
    } 
    else {
      if ((last_click_y-y)>2) {
	kp *= 1.3;
	last_click_y = y;
      } 
    }
    if (kp > kp_max) kp = kp_max;
    if (kp < 0.0) kp = 0.0;
  } else last_click_y = y;last_click_x = x;
  refresh_param_subwindows();
}

void cont_mouse_motion(int x, int y)
  // Callback for mouse drags in the Bode window
{
  if (adjusting_klf)
    {
      if (last_click_y >= 0) {
	if ((y-last_click_y)>2) {
	  klf /= 1.1;
	  last_click_y = y;
	} 
	else {
	  if ((last_click_y-y)>2) {
	    klf *= 1.1;
	    last_click_y = y;
	  } 
	}
	if (klf > 30.0) klf = 30.0;
	if (klf < 0.03) klf = 0.03;
	refresh_param_subwindows();
      } else 
	{
	  last_click_y = y;
	  last_click_x=x;
	}
    }
  if (adjusting_om_max)
    {
      if (last_click_y >= 0) {
	if ((x-last_click_x)>2) {
	  om_max *= 1.07;
	  last_click_x = x;
	} 
	else {
	  if ((last_click_x-x)>2) {
	    om_max /= 1.07;
	    last_click_x = x;
 	  } 
	}
	if (om_max > 10.0) om_max = 10.0;
	if (om_max < 0.010) om_max = 0.010;
	refresh_param_subwindows();
      } else 
	{
	  last_click_y = y;
	  last_click_x=x;
	}
    }
}

void glut_special(int key, int x, int y)
  // Callback for special key presses in all windows
{
  switch(key) {
  case GLUT_KEY_UP:
  case GLUT_KEY_RIGHT:
    millisecond_delay -= 1;
    if (millisecond_delay < 0) millisecond_delay = 0;
    break;
  case GLUT_KEY_DOWN:
  case GLUT_KEY_LEFT:
    millisecond_delay += 1;
    break;
  }
}

void glut_key (unsigned char k, int x, int y)
  // Callback for key presses in all windows
{
  int i;
  switch(k) {
    
  case 27: case 'q': case 'Q':
    // Escape or q or Q  - exit
    exit(0);
    break;

  case 's': case 'S':
    // s or S
    if (sinusoid) {
      sinusoid = false;
      signal_voltage = 0.0;
    }
    else sinusoid = true;
    refresh_param_subwindows();    
break; 

  case 'c': case 'C':
    // c or C
    if (controller) {
      controller = false;
    }
    else {
      controller = true;
      draw_cont_window();
    }
    refresh_param_subwindows();    
break; 

  case 'z': case 'Z':
  // Zero the time traces
  for (i=0; i<400; i++) {
    historical_ref[i] = 0.0;
    historical_yout[i] = 0.0;
    historical_u[i] = 0.0;
    historical_error[i] = 0.0;
    signal_voltage=0.0;
  }
break; 

  case 32:
    // Space bar
    if (paused) {
      //      glutIdleFunc(draw_live_windows);
      paused = false;
      glutTimerFunc(0,draw_live_windows,0);
    } else {
      //      glutIdleFunc(NULL);
      paused = true;
    }
    break;

  }
}

int main (int argc, char* argv[])
{
  int i;
  complex<float> z;
  complex<float> jomegat;
  // Main GLUT window
  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
  glutInitWindowPosition(0, 0);
  glutInitWindowSize(PREFERRED_WIDTH, PREFERRED_HEIGHT);
  subwin_width = (PREFERRED_WIDTH - 6*GAP)/3;
  subwin_height = (PREFERRED_HEIGHT - 6*GAP)/3;
  main_window = glutCreateWindow("Virtual Control Lab (Glenn Vinnicombe, November 2006)");
  glDrawBuffer(GL_BACK);
  glLineWidth(2.0);
  glDisable(GL_LIGHTING);
  glDisable(GL_DEPTH_TEST);
  glutDisplayFunc(draw_main_window);
  glutReshapeFunc(reshape_main_window);  
  //  glutIdleFunc(draw_live_windows);
  glutTimerFunc(1,draw_live_windows,0);
  glutTimerFunc(1,really_refresh_param_subwindows,0);
  glutKeyboardFunc(glut_key);
  glutSpecialFunc(glut_special);
  glutMotionFunc(NULL);
  glutPassiveMotionFunc(NULL);
  //  glutFullScreen();


  // The characteristic subwindow
  characteristic_window = glutCreateSubWindow(main_window,0,0,subwin_width,subwin_height);
  glutDisplayFunc(draw_characteristic_window);
  glutKeyboardFunc(glut_key);
  glutSpecialFunc(glut_special);

  // The Ref subwindow
  ref_window = glutCreateSubWindow(main_window, 0,0, subwin_width, subwin_height);
  glutDisplayFunc(draw_ref_window);
  glutMouseFunc(ref_mouse_button);
  glutMotionFunc(ref_mouse_motion);
  glutKeyboardFunc(glut_key);
  glutSpecialFunc(glut_special);

  // The Error subwindow
  error_window = glutCreateSubWindow(main_window, 0,0, subwin_width, subwin_height);
  glutDisplayFunc(draw_error_window);
  glutKeyboardFunc(glut_key);
  glutSpecialFunc(glut_special);

  // The Nyquist subwindow
  nyquist_window = glutCreateSubWindow(main_window, 0,0, subwin_width, subwin_height);
  glutDisplayFunc(draw_nyquist_window);
  glutMouseFunc(nyquist_mouse_button);
  glutMotionFunc(nyquist_mouse_motion);
  glutKeyboardFunc(glut_key);
  glutSpecialFunc(glut_special);


  // The Bode subwindow
  bode_window = glutCreateSubWindow(main_window, 0,0, subwin_width, subwin_height);
  glutDisplayFunc(draw_bode_window);
  glutMouseFunc(bode_mouse_button);
  glutMotionFunc(bode_mouse_motion);
  glutKeyboardFunc(glut_key);
  glutSpecialFunc(glut_special);

  // The Cont subwindow
  cont_window = glutCreateSubWindow(main_window, 0,0, subwin_width, subwin_height);
  glutDisplayFunc(draw_cont_window);
  glutMouseFunc(cont_mouse_button);
  glutMotionFunc(cont_mouse_motion);
  glutKeyboardFunc(glut_key);
  glutSpecialFunc(glut_special);

  // The Clfr subwindow
  clfr_window = glutCreateSubWindow(main_window, 0,0, subwin_width, subwin_height);
  glutDisplayFunc(draw_clfr_window);
  glutKeyboardFunc(glut_key);
  glutSpecialFunc(glut_special);

  // The Instructions subwindow
  inst_window = glutCreateSubWindow(main_window, 0,0, subwin_width, subwin_height);
  glutDisplayFunc(draw_inst_window);
  glutKeyboardFunc(glut_key);
  glutSpecialFunc(glut_special);
  glutMotionFunc(NULL);
  glutPassiveMotionFunc(NULL);

  // The circuit subwindow
  circuit_window = glutCreateSubWindow(main_window, 0,0, subwin_width, subwin_height);
  glutDisplayFunc(draw_circuit_window);
  glutMouseFunc(circuit_mouse_button);
  glutMotionFunc(circuit_mouse_motion);
  glutKeyboardFunc(glut_key);
  glutSpecialFunc(glut_special);

  // Zero the time traces
  for (i=0; i<400; i++) {
    historical_ref[i] = 0.0;
    historical_yout[i] = 0.0;
    historical_u[i] = 0.0;
    historical_error[i] = 0.0;
  }
  // Set up the model and calculate its frequency response
    den[0]=1;den[1]=-2.7997;den[2]=2.6104;den[3]=-.8106;
    num[0]=.1582e-3;num[1]=.6005e-3;num[2]=.1424e-3;
    //  den[0]=1;den[1]=-2.8097;den[2]=2.6284;den[3]=-.8187;
    // num[0]=.1586e-3;num[1]=.6035e-3;num[2]=.1435e-3;
  for (i=0; i<151; i++) {
    if(i>0){
      jomegat=pow(10.0,(i-100.0)/50.0)*delta_t;
      jomegat=jomegat*sqrt((complex<float>)-1);
      z=exp(jomegat);
    }
    else z=1;
    frsp[i]=((complex<float>)num[0]*pow(z,2)+(complex<float>)num[1]*pow(z,1)+(complex<float>)num[2])/((complex<float>)den[0]*pow(z,3)+(complex<float>)den[1]*pow(z,2)+(complex<float>)den[2]*pow(z,1)+(complex<float>)den[3]);
    frsp_ph[i]=arg(frsp[i])*180.0/3.1415927;
    if (frsp_ph[i]>0) frsp_ph[i]=frsp_ph[i]-360;
    frsp_mag[i]=20.0*log10(abs(frsp[i]));
    frsp_real[i]=real(frsp[i]);
    frsp_imag[i]=imag(frsp[i]);
  }

  // User instructions
  cout << "Virtual Control Lab, Glenn Vinnicombe, November 2006" << endl;
  cout << "---------------------------------------------------" << endl << endl;
  cout << "Anything in magenta is \"click and dragable\"" << endl;
  cout << "Click and wiggle in the middle left window to set a reference." << endl;
  cout << "Press \"s\" to toggle on/off a sinusoidal reference signal." << endl;
  cout << "Click and drag (up and down) on the magenta numbers in the top left figure to change the gain and the value/frequency of the reference signal." << endl;
  cout << "Press the space bar to pause the scrolling traces." << endl;
  cout << "Use the cursor keys to adjust the speed of the scrolling traces." << endl;
  cout << "Press q to quit." << endl;

  glutMainLoop();
}
